@@ -1,6 +1,7 @@
 {-# LANGUAGE BangPatterns, CPP, DeriveDataTypeable, FlexibleContexts,
-    MagicHash, Rank2Types, ScopedTypeVariables, TypeFamilies, UnboxedTuples,
-    ForeignFunctionInterface #-}
+    FlexibleInstances, MultiParamTypeClasses, MagicHash, Rank2Types,
+    ScopedTypeVariables, TypeFamilies, UnboxedTuples
+    #-}
 -- |
 -- Module    : System.Random.MWC
 -- Copyright : (c) 2009-2012 Bryan O'Sullivan
@@ -10,68 +11,124 @@
 -- Stability   : experimental
 -- Portability : portable
 --
--- Pseudo-random number generation.  This module contains code for
--- generating high quality random numbers that follow a uniform
--- distribution.
+-- Pseudo-random number generation using Marsaglia's MWC256, (also
+-- known as MWC8222) multiply-with-carry generator, which has a period
+-- of \(2^{8222}\) and fares well in tests of randomness.  It is also
+-- extremely fast, between 2 and 3 times faster than the Mersenne
+-- Twister. There're two representation of generator: 'Gen' which is
+-- generator that uses in-place mutation and 'Seed' which is immutable
+-- snapshot of generator's state.
 --
--- For non-uniform distributions, see the
--- 'System.Random.MWC.Distributions' module.
 --
--- The uniform PRNG uses Marsaglia's MWC256 (also known as MWC8222)
--- multiply-with-carry generator, which has a period of 2^8222 and
--- fares well in tests of randomness.  It is also extremely fast,
--- between 2 and 3 times faster than the Mersenne Twister.
+-- == Initialization
 --
--- The generator state is stored in the 'Gen' data type. It can be
--- created in several ways:
+-- Generator could be initialized in several ways. One is to obtain
+-- randomness from operating system using 'createSystemRandom',
+-- 'createSystemSeed' or 'withSystemRandomST' (All examples assume
+-- that @System.Random.Stateful@ is imported)
 --
---   1. Using the 'withSystemRandom' call, which creates a random state.
+-- >>> g <- createSystemRandom
+-- >>> uniformM g :: IO Int
+-- ...
 --
---   2. Supply your own seed to 'initialize' function.
+-- >>> withSystemRandomST $ \g -> uniformM g :: IO Int
+-- ...
 --
---   3. Finally, 'create' makes a generator from a fixed seed.
---      Generators created in this way aren't really random.
+-- Deterministically create generator from given seed using
+-- 'initialize' function:
+--
+-- >>> import Data.Int
+-- >>> import qualified Data.Vector.Unboxed as U
+-- >>> import System.Random.Stateful
+-- >>> g <- initialize $ U.fromList [1,2,3]
+-- >>> uniformRM (1,200) g :: IO Int64
+-- 101
+--
+-- Last way is to create generator with fixed seed which could be
+-- useful in testing
+--
+-- >>> g <- create
+-- >>> uniformM g :: IO Int
+-- -8765701622605876598
+--
+--
+-- == Generation of random numbers
+--
+-- Recommended way of generating random numbers in simple cases like
+-- generating uniformly distributed random number in range or value
+-- uniformly distributed in complete type domain is to use
+-- 'UniformRange' and 'Uniform' type classes. Note that while small
+-- self-contained examples usually require explicit annotations
+-- usually result type could be inferred.
+--
+-- This example simulates 20 throws of fair 6-sided dice:
+--
+-- >>> g <- create
+-- >>> replicateM 20 $ uniformRM (1, 6::Integer) g
+-- [3,4,3,1,4,6,1,6,1,4,2,2,3,2,4,2,5,1,3,5]
+--
+-- For generating full range of possible values one could use
+-- 'uniformM'. This example generates 10 random bytes, or equivalently
+-- 10 throws of 256-sided dice:
+--
+-- >>> g <- create
+-- >>> replicateM 10 $ uniformM g :: IO [Word8]
+-- [209,138,126,150,165,15,69,203,155,146]
+--
+-- There're special functions for generation of @Doubles@ and @Float
+-- in unit interval: 'Random.uniformDouble01M',
+-- 'Random.uniformDoublePositive01M', 'Random.uniformFloat01M',
+-- 'Random.uniformFloatPositive01M':
+--
+-- >>> uniformDouble01M =<< create
+-- 0.5248103628705498
+-- >>> uniformFloat01M =<< create
+-- 0.5248104
+--
+-- For normal distribution and others see modules
+-- "System.Random.MWC.Distributions" and
+-- "System.Random.MWC.CondensedTable". Note that they could be used
+-- with any other generator implementing 'Random.StatefulGen' API
+--
+-- There're special cases for generating random vectors and
+-- bytestrings. For example in order to generate random 10-byte
+-- sequences as unboxed vector or bytestring:
+--
+-- >>> g <- create
+-- >>> uniformVector g 10 :: IO (U.Vector Word8)
+-- [209,138,126,150,165,15,69,203,155,146]
+--
+-- >>> import qualified Data.ByteString as BS
+-- >>> g <- create
+-- >>> BS.unpack <$> uniformByteStringM 10 g
+-- [138,242,130,33,209,248,89,134,150,180]
+--
+-- Note that 'Random.uniformByteStringM' produces different result
+-- from 'uniformVector' since it uses PRNG's output more efficently.
+--
+--
+-- == State handling
 --
 -- For repeatability, the state of the generator can be snapshotted
--- and replayed using the 'save' and 'restore' functions.
+-- and replayed using the 'save' and 'restore' functions. Following
+-- example shows how to save and restore generator:
 --
--- The simplest use is to generate a vector of uniformly distributed values:
---
--- @
---   vs \<- 'withSystemRandom' . 'asGenST' $ \\gen -> 'uniformVector' gen 100
--- @
---
--- These values can be of any type which is an instance of the class
--- 'Variate'.
---
--- To generate random values on demand, first 'create' a random number
--- generator.
---
--- @
---   gen <- 'create'
--- @
---
--- Hold onto this generator and use it wherever random values are
--- required (creating a new generator is expensive compared to
--- generating a random number, so you don't want to throw them
--- away). Get a random value using 'uniform' or 'uniformR':
---
--- @
---   v <- 'uniform' gen
--- @
---
--- @
---   v <- 'uniformR' (1, 52) gen
--- @
+-- >>> g <- create
+-- >>> replicateM_ 10 (uniformM g :: IO Word64)
+-- >>> s <- save g
+-- >>> uniformM g :: IO Word32
+-- 1771812561
+-- >>> uniformM =<< restore s :: IO Word32
+-- 1771812561
 module System.Random.MWC
     (
     -- * Gen: Pseudo-Random Number Generators
       Gen
     , create
     , initialize
-    , withSystemRandom
+    , createSystemSeed
     , createSystemRandom
-
+    , withSystemRandomST
     -- ** Type helpers
     -- $typehelp
     , GenIO
@@ -80,6 +137,8 @@ module System.Random.MWC
     , asGenST
 
     -- * Variates: uniformly distributed values
+    , Random.Uniform(..)
+    , Random.UniformRange(..)
     , Variate(..)
     , uniformVector
 
@@ -89,7 +148,8 @@ module System.Random.MWC
     , toSeed
     , save
     , restore
-
+    -- * Deprecated
+    , withSystemRandom
     -- * References
     -- $references
     ) where
@@ -99,38 +159,28 @@ module System.Random.MWC
 #endif
 
 import Control.Monad           (ap, liftM, unless)
-import Control.Monad.Primitive (PrimMonad, PrimState, unsafePrimToIO)
-#if MIN_VERSION_primitive(0,6,0)
-import Control.Monad.Primitive (PrimBase)
-#endif
-import Control.Monad.ST        (ST)
+import Control.Monad.Primitive (PrimMonad, PrimBase, PrimState, unsafePrimToIO, unsafeSTToPrim)
+import Control.Monad.ST        (ST,runST)
 import Data.Bits               ((.&.), (.|.), shiftL, shiftR, xor)
 import Data.Int                (Int8, Int16, Int32, Int64)
-import Data.IORef              (atomicModifyIORef, newIORef)
-import Data.Ratio              ((%), numerator)
-import Data.Time.Clock.POSIX   (getPOSIXTime)
+import Data.IORef              (IORef, atomicModifyIORef, newIORef)
 import Data.Typeable           (Typeable)
 import Data.Vector.Generic     (Vector)
-import Data.Word               (Word8, Word16, Word32, Word64)
-#if !MIN_VERSION_base(4,8,0)
-import Data.Word               (Word)
-#endif
-import Foreign.Marshal.Alloc   (allocaBytes)
-import Foreign.Marshal.Array   (peekArray)
+import Data.Word
 import qualified Data.Vector.Generic         as G
+import qualified Data.Vector.Generic.Mutable as GM
 import qualified Data.Vector.Unboxed         as I
 import qualified Data.Vector.Unboxed.Mutable as M
-import System.CPUTime   (cpuTimePrecision, getCPUTime)
-import System.IO        (IOMode(..), hGetBuf, hPutStrLn, stderr, withBinaryFile)
+import System.IO        (hPutStrLn, stderr)
 import System.IO.Unsafe (unsafePerformIO)
 import qualified Control.Exception as E
-#if defined(mingw32_HOST_OS)
-import Foreign.Ptr
-import Foreign.C.Types
-#endif
+import System.Random.MWC.SeedSource
+import qualified System.Random.Stateful as Random
 
-
--- | The class of types for which we can generate uniformly
+-- | NOTE: Consider use of more principled type classes
+-- 'Random.Uniform' and 'Random.UniformRange' instead.
+--
+-- The class of types for which we can generate uniformly
 -- distributed random variates.
 --
 -- The uniform PRNG uses Marsaglia's MWC256 (also known as MWC8222)
@@ -202,7 +252,7 @@ instance Variate Word16 where
     {-# INLINE uniformR #-}
 
 instance Variate Word32 where
-    uniform  = uniform1 fromIntegral
+    uniform  = uniform1 id
     uniformR a b = uniformRange a b
     {-# INLINE uniform  #-}
     {-# INLINE uniformR #-}
@@ -235,33 +285,29 @@ instance Variate Double where
     {-# INLINE uniformR #-}
 
 instance Variate Int where
-#if WORD_SIZE_IN_BITS < 64
+#if WORD_SIZE_IN_BITS == 32
     uniform = uniform1 fromIntegral
-#else
+#elif WORD_SIZE_IN_BITS == 64
     uniform = uniform2 wordsTo64Bit
+#else
+#error "Word size is not 32 nor 64"
 #endif
     uniformR a b = uniformRange a b
     {-# INLINE uniform  #-}
     {-# INLINE uniformR #-}
 
 instance Variate Word where
-#if WORD_SIZE_IN_BITS < 64
+#if WORD_SIZE_IN_BITS == 32
     uniform = uniform1 fromIntegral
-#else
+#elif WORD_SIZE_IN_BITS == 64
     uniform = uniform2 wordsTo64Bit
+#else
+#error "Word size is not 32 nor 64"
 #endif
     uniformR a b = uniformRange a b
     {-# INLINE uniform  #-}
     {-# INLINE uniformR #-}
 
-{-
-instance Variate Integer where
-    uniform g = do
-      u <- uniform g
-      return $! fromIntegral (u :: Int)
-    {-# INLINE uniform #-}
--}
-
 instance (Variate a, Variate b) => Variate (a,b) where
     uniform g = (,) `liftM` uniform g `ap` uniform g
     uniformR ((x1,y1),(x2,y2)) g = (,) `liftM` uniformR (x1,x2) g `ap` uniformR (y1,y2) g
@@ -359,6 +405,17 @@ create = initialize defaultSeed
 -- the following example:
 --
 -- @gen' <- 'initialize' . 'fromSeed' =<< 'save'@
+--
+-- In the MWC algorithm, the /carry/ value must be strictly smaller than the
+-- multiplicator (see https://en.wikipedia.org/wiki/Multiply-with-carry).
+-- Hence, if a seed contains exactly 258 elements, the /carry/ value, which is
+-- the last of the 258 values, is moduloed by the multiplicator.
+--
+-- Note that if the /first/ carry value is strictly smaller than the multiplicator,
+-- all subsequent carry values are also strictly smaller than the multiplicator
+-- (a proof of this is in the comments of the code of 'uniformWord32'), hence
+-- when restoring a saved state, we have the guarantee that moduloing the saved
+-- carry won't modify its value.
 initialize :: (PrimMonad m, Vector v Word32) =>
               v Word32 -> m (Gen (PrimState m))
 initialize seed = do
@@ -367,7 +424,7 @@ initialize seed = do
     if fini == 258
       then do
         M.unsafeWrite q ioff $ G.unsafeIndex seed ioff .&. 255
-        M.unsafeWrite q coff $ G.unsafeIndex seed coff
+        M.unsafeWrite q coff $ G.unsafeIndex seed coff `mod` fromIntegral aa
       else do
         M.unsafeWrite q ioff 255
         M.unsafeWrite q coff 362436
@@ -384,11 +441,35 @@ initialize seed = do
 {-# INLINE initialize #-}
 
 -- | An immutable snapshot of the state of a 'Gen'.
-newtype Seed = Seed {
-    -- | Convert seed into vector.
-    fromSeed :: I.Vector Word32
-    }
-    deriving (Eq, Show, Typeable)
+newtype Seed = Seed (I.Vector Word32)
+  deriving (Eq, Show, Typeable)
+
+-- | Convert seed into vector.
+fromSeed :: Seed -> I.Vector Word32
+fromSeed (Seed v) = v
+
+-- | @since 0.15.0.0
+instance (s ~ PrimState m, PrimMonad m) => Random.StatefulGen (Gen s) m where
+  uniformWord32R u = uniformR (0, u)
+  {-# INLINE uniformWord32R #-}
+  uniformWord64R u = uniformR (0, u)
+  {-# INLINE uniformWord64R #-}
+  uniformWord8 = uniform
+  {-# INLINE uniformWord8 #-}
+  uniformWord16 = uniform
+  {-# INLINE uniformWord16 #-}
+  uniformWord32 = uniform
+  {-# INLINE uniformWord32 #-}
+  uniformWord64 = uniform
+  {-# INLINE uniformWord64 #-}
+  uniformShortByteString n g = unsafeSTToPrim (Random.genShortByteStringST n (uniform g))
+  {-# INLINE uniformShortByteString #-}
+
+-- | @since 0.15.0.0
+instance PrimMonad m => Random.FrozenGen Seed m where
+  type MutableGen Seed m = Gen (PrimState m)
+  thawGen = restore
+  freezeGen = save
 
 -- | Convert vector to 'Seed'. It acts similarily to 'initialize' and
 -- will accept any vector. If you want to pass seed immediately to
@@ -409,105 +490,73 @@ restore (Seed s) = Gen `liftM` G.thaw s
 {-# INLINE restore #-}
 
 
--- Aquire seed from current time. This is horrible fallback for
--- Windows system.
-acquireSeedTime :: IO [Word32]
-acquireSeedTime = do
-  c <- (numerator . (%cpuTimePrecision)) `liftM` getCPUTime
-  t <- toRational `liftM` getPOSIXTime
-  let n    = fromIntegral (numerator t) :: Word64
-  return [fromIntegral c, fromIntegral n, fromIntegral (n `shiftR` 32)]
+-- $seeding
+--
+-- Library provides several functions allowing to intialize generator
+-- using OS-provided randomness: \"@\/dev\/urandom@\" on Unix-like
+-- systems or @RtlGenRandom@ on Windows. This is a somewhat expensive
+-- function, and is intended to be called only occasionally (e.g. once
+-- per thread).  You should use the `Gen` it creates to generate many
+-- random numbers.
 
--- | Acquire seed from the system entropy source. On Unix machines,
--- this will attempt to use @/dev/urandom@. On Windows, it will internally
--- use @RtlGenRandom@.
-acquireSeedSystem :: IO [Word32]
-acquireSeedSystem = do
-#if !defined(mingw32_HOST_OS)
-  -- Read 256 random Word32s from /dev/urandom
-  let nbytes = 1024
-      random = "/dev/urandom"
-  allocaBytes nbytes $ \buf -> do
-    nread <- withBinaryFile random ReadMode $
-               \h -> hGetBuf h buf nbytes
-    peekArray (nread `div` 4) buf
-#else
-  let nbytes = 1024
-  -- Generate 256 random Word32s from RtlGenRandom
-  allocaBytes nbytes $ \buf -> do
-    ok <- c_RtlGenRandom buf (fromIntegral nbytes)
-    if ok then return () else fail "Couldn't use RtlGenRandom"
-    peekArray (nbytes `div` 4) buf
-
--- Note: on 64-bit Windows, the 'stdcall' calling convention
--- isn't supported, so we use 'ccall' instead.
-#if defined(i386_HOST_ARCH)
-# define WINDOWS_CCONV stdcall
-#elif defined(x86_64_HOST_ARCH)
-# define WINDOWS_CCONV ccall
-#else
-# error Unknown mingw32 architecture!
-#endif
-
--- Note: On Windows, the typical convention would be to use
--- the CryptoGenRandom API in order to generate random data.
--- However, here we use 'SystemFunction036', AKA RtlGenRandom.
---
--- This is a commonly used API for this purpose; one bonus is
--- that it avoids having to bring in the CryptoAPI library,
--- and completely sidesteps the initialization cost of CryptoAPI.
---
--- While this function is technically "subject to change" that is
--- extremely unlikely in practice: rand_s in the Microsoft CRT uses
--- this, and they can't change it easily without also breaking
--- backwards compatibility with e.g. statically linked applications.
---
--- The name 'SystemFunction036' is the actual link-time name; the
--- display name is just for giggles, I guess.
---
--- See also:
---   - http://blogs.msdn.com/b/michael_howard/archive/2005/01/14/353379.aspx
---   - https://bugzilla.mozilla.org/show_bug.cgi?id=504270
---
-foreign import WINDOWS_CCONV unsafe "SystemFunction036"
-  c_RtlGenRandom :: Ptr a -> CULong -> IO Bool
-#endif
-
--- | Seed a PRNG with data from the system's fast source of
--- pseudo-random numbers (\"@\/dev\/urandom@\" on Unix-like systems or
--- @RtlGenRandom@ on Windows), then run the given action.
---
--- This is a somewhat expensive function, and is intended to be called
--- only occasionally (e.g. once per thread).  You should use the `Gen`
--- it creates to generate many random numbers.
-withSystemRandom ::
-#if MIN_VERSION_primitive(0,6,0)
-                    PrimBase m
-#else
-                    PrimMonad m
-#endif
-                 => (Gen (PrimState m) -> m a) -> IO a
-withSystemRandom act = do
-  seed <- acquireSeedSystem `E.catch` \(_::E.IOException) -> do
-    seen <- atomicModifyIORef warned ((,) True)
+createSystemRandomList :: IO [Word32]
+createSystemRandomList = do
+  acquireSeedSystem 256 `E.catch` \(_::E.IOException) -> do
+    seen <- atomicModifyIORef seedCreatetionWarned ((,) True)
     unless seen $ E.handle (\(_::E.IOException) -> return ()) $ do
-#if !defined(mingw32_HOST_OS)
-      hPutStrLn stderr ("Warning: Couldn't open /dev/urandom")
-#else
-      hPutStrLn stderr ("Warning: Couldn't use RtlGenRandom")
-#endif
+      hPutStrLn stderr $ "Warning: Couldn't use randomness source " ++ randomSourceName
       hPutStrLn stderr ("Warning: using system clock for seed instead " ++
                         "(quality will be lower)")
     acquireSeedTime
-  unsafePrimToIO $ initialize (I.fromList seed) >>= act
-  where
-    warned = unsafePerformIO $ newIORef False
-    {-# NOINLINE warned #-}
 
--- | Seed a PRNG with data from the system's fast source of pseudo-random
--- numbers. All the caveats of 'withSystemRandom' apply here as well.
+seedCreatetionWarned :: IORef Bool
+seedCreatetionWarned = unsafePerformIO $ newIORef False
+{-# NOINLINE seedCreatetionWarned #-}
+
+
+
+-- | Generate random seed for generator using system's fast source of
+--   pseudo-random numbers.
+--
+-- @since 0.15.0.0
+createSystemSeed :: IO Seed
+createSystemSeed = do
+  seed <- createSystemRandomList
+  return $! toSeed $ I.fromList seed
+
+-- | Seed a PRNG with data from the system's fast source of
+--   pseudo-random numbers.
 createSystemRandom :: IO GenIO
-createSystemRandom = withSystemRandom (return :: GenIO -> IO GenIO)
+createSystemRandom = initialize . I.fromList =<< createSystemRandomList
+
+
+-- | Seed PRNG with data from the system's fast source of
+--   pseudo-random numbers and execute computation in ST monad.
+--
+-- @since 0.15.0.0
+withSystemRandomST :: (forall s. Gen s -> ST s a) -> IO a
+withSystemRandomST act = do
+  seed <- createSystemSeed
+  return $! runST $ act =<< restore seed
+
+-- | Seed a PRNG with data from the system's fast source of
+--   pseudo-random numbers, then run the given action.
+--
+--   This function is unsafe and for example allows STRefs or any
+--   other mutable data structure to escape scope:
+--
+--   >>> ref <- withSystemRandom $ \_ -> newSTRef 1
+--   >>> withSystemRandom $ \_ -> modifySTRef ref succ >> readSTRef ref
+--   2
+--   >>> withSystemRandom $ \_ -> modifySTRef ref succ >> readSTRef ref
+--   3
+withSystemRandom :: PrimBase m
+                 => (Gen (PrimState m) -> m a) -> IO a
+withSystemRandom act = do
+  seed <- createSystemSeed
+  unsafePrimToIO $ act =<< restore seed
+{-# DEPRECATED withSystemRandom "Use withSystemRandomST or createSystemSeed or createSystemRandom instead" #-}
+
 
 -- | Compute the next index into the state pool.  This is simply
 -- addition modulo 256.
@@ -516,11 +565,16 @@ nextIndex i = fromIntegral j
     where j = fromIntegral (i+1) :: Word8
 {-# INLINE nextIndex #-}
 
+-- The multiplicator : 0x5BCF5AB2
+--
+-- Eventhough it is a 'Word64', it is important for the correctness of the proof
+-- on carry value that it is /not/ greater than maxBound 'Word32'.
 aa :: Word64
 aa = 1540315826
 {-# INLINE aa #-}
 
 uniformWord32 :: PrimMonad m => Gen (PrimState m) -> m Word32
+-- NOTE [Carry value]
 uniformWord32 (Gen q) = do
   i  <- nextIndex `liftM` M.unsafeRead q ioff
   c  <- fromIntegral `liftM` M.unsafeRead q coff
@@ -532,7 +586,7 @@ uniformWord32 (Gen q) = do
                      | otherwise = (# x,     c' #)
   M.unsafeWrite q i x'
   M.unsafeWrite q ioff (fromIntegral i)
-  M.unsafeWrite q coff (fromIntegral c'')
+  M.unsafeWrite q coff c''
   return x'
 {-# INLINE uniformWord32 #-}
 
@@ -562,7 +616,7 @@ uniform2 f (Gen q) = do
   M.unsafeWrite q i x'
   M.unsafeWrite q j y'
   M.unsafeWrite q ioff (fromIntegral j)
-  M.unsafeWrite q coff (fromIntegral d'')
+  M.unsafeWrite q coff d''
   return $! f x' y'
 {-# INLINE uniform2 #-}
 
@@ -581,24 +635,8 @@ type instance Unsigned Word16 = Word16
 type instance Unsigned Word32 = Word32
 type instance Unsigned Word64 = Word64
 
--- This is workaround for bug #25.
---
--- GHC-7.6 has a bug (#8072) which results in calculation of wrong
--- number of buckets in function `uniformRange'. Consequently uniformR
--- generates values in wrong range.
---
--- Bug only affects 32-bit systems and Int/Word data types. Word32
--- works just fine. So we set Word32 as unsigned counterpart for Int
--- and Word on 32-bit systems. It's done only for GHC-7.6 because
--- other versions are unaffected by the bug and we expect that GHC may
--- optimise code which uses Word better.
-#if (WORD_SIZE_IN_BITS < 64) && (__GLASGOW_HASKELL__ == 706)
-type instance Unsigned Int   = Word32
-type instance Unsigned Word  = Word32
-#else
 type instance Unsigned Int   = Word
 type instance Unsigned Word  = Word
-#endif
 
 
 -- Subtract two numbers under assumption that x>=y and store result in
@@ -640,12 +678,25 @@ uniformRange (x1,x2) g
 -- | Generate a vector of pseudo-random variates.  This is not
 -- necessarily faster than invoking 'uniform' repeatedly in a loop,
 -- but it may be more convenient to use in some situations.
-uniformVector :: (PrimMonad m, Variate a, Vector v a)
-             => Gen (PrimState m) -> Int -> m (v a)
-uniformVector gen n = G.replicateM n (uniform gen)
+uniformVector
+  :: (PrimMonad m, Random.StatefulGen g m, Random.Uniform a, Vector v a)
+  => g -> Int -> m (v a)
+-- NOTE: We use in-place mutation in order to generate vector instead
+--       of generateM because latter will go though intermediate list until
+--       we're working in IO/ST monad
+--
+-- See: https://github.com/haskell/vector/issues/208 for details
+uniformVector gen n = do
+  mu <- GM.unsafeNew n
+  let go !i | i < n     = Random.uniformM gen >>= GM.unsafeWrite mu i >> go (i+1)
+            | otherwise = G.unsafeFreeze mu
+  go 0
 {-# INLINE uniformVector #-}
 
 
+-- This is default seed for the generator and used when no seed is
+-- specified or seed is only partial. It's not known how it was
+-- generated but it looks random enough
 defaultSeed :: I.Vector Word32
 defaultSeed = I.fromList [
   0x7042e8b3, 0x06f7f4c5, 0x789ea382, 0x6fb15ad8, 0x54f7a879, 0x0474b184,
@@ -744,3 +795,40 @@ defaultSeed = I.fromList [
 --
 -- This is almost as compact as the original code that the compiler
 -- rejected.
+
+
+
+-- $setup
+--
+-- >>> import Control.Monad
+-- >>> import Data.Word
+-- >>> import Data.STRef
+-- >>> :set -Wno-deprecations
+
+
+-- NOTE [Carry value]
+-- ------------------
+-- This is proof of statement:
+--
+-- > if the carry value is strictly smaller than the multiplicator,
+-- > the next carry value is also strictly smaller than the multiplicator.
+--
+-- Eventhough the proof is written in terms of the actual value of the
+-- multiplicator, it holds for any multiplicator value /not/ greater
+-- than maxBound 'Word32'
+--
+--    (In the code, the multiplicator is aa, the carry value is c,
+--     the next carry value is c''.)
+--
+-- So we'll assume that c < aa, and show that c'' < aa :
+--
+-- by definition, aa = 0x5BCF5AB2, qi <= 0xFFFFFFFF (because it is a 'Word32')
+--
+-- Then we get following:
+--
+--    aa*qi <= 0x5BCF5AB200000000 - 0x5BCF5AB2.
+--    t     <  0x5BCF5AB200000000 (because t = aa * qi + c and c < 0x5BCF5AB2)
+--    t     <= 0x5BCF5AB1FFFFFFFF
+--    c'    <  0x5BCF5AB1
+--    c''   <  0x5BCF5AB2,
+--    c''   < aa, which is what we wanted to prove.
